(* Script moves every mail from the IMAP-folders of your active mail accounts to ON MY MAC, creates mailboxes and folders as necessary *)

global gArchiveMailboxName, gMinimumAgeDays, gCurrentDate

(* UnMain() *)
on run (argv)
	(* Globals *)
	set gArchiveMailboxName to "Archive"
	set gMinimumAgeDays to 30 * 3 # three months
	tell AppleScript to set gCurrentDate to current date
	
	(* check if "Archive"-Mailbox exists, otherwise create it! *)
	tell application "Mail"
		if not (exists mailbox gArchiveMailboxName) then
			set mbox to make new mailbox with properties {name:gArchiveMailboxName}
		end if
	end tell
	
	(* Tell mail what to do *)
	tell application "Mail"
		
		(* Get the defined accounts *)
		set accountsArray to every account
		set accountsCount to (count of accounts)
		
		(* More than 0 accounts? *)
		if (accountsCount is greater than 0) then
			
			(* Walk through accounts *)
			repeat with a from 1 to accountsCount
				
				(* Get characteristics of account *)
				set aAccount to item a of accounts
				set aAccountName to name of aAccount
				if (enabled of aAccount is true) then
					
					set boxes to mailboxes of aAccount
					log boxes
					
					(* Walk thru mailbox of account *)
					repeat with box in boxes
						set accountMailboxName to name of box
						log name of box
						
						(* Check if an equivalent mailbox exists in ON MY MAC, otherwise create it *)
						tell application "Mail"
							if not (exists mailbox (gArchiveMailboxName & "/" & accountMailboxName)) then
								set mbox to make new mailbox with properties {name:gArchiveMailboxName & "/" & accountMailboxName}
							end if
						end tell
						
						(* Move the messages *)
						if true then
							(* Move all mails older than n days *)
							
							(* Walk through messages *)
							repeat with aMessage in (messages of box)
								try
									set messageSentDate to date sent of aMessage
									#log "Message date: " & messageSentDate
									
									(* Check age of that date *)
									set currentDate to gCurrentDate -- current date
									set currentDateFoo to currentDate - (gMinimumAgeDays * days)
									#log "currentDateFoo: " & currentDateFoo
									if (currentDateFoo > messageSentDate) then (* Attention: "comes before" and "less than" does NOT work, only "<", see Erik's page *)
										(* Move the mail from Account's folder (IMAP) to local (ON MY MAC) *)
										tell application "Mail"
											move {aMessage} to mailbox (gArchiveMailboxName & "/" & accountMailboxName) (* http://hintsforums.macworld.com/archive/index.php/t-81889.html *)
										end tell
									end if
								on error (e)
									log (e)
								end try
							end repeat (* repeat with messages *)
						end if (* enabled? *)
						
						(* Move the messages *)
						if false then
							(* Move all mails older than n days *)
							with timeout of 300 seconds
								set messagesArray to messages of box
							end timeout
							set messagesCount to count of messagesArray
							log "Found " & messagesCount & " messages..."
							
							(* Check how much we must now process *)
							if (messagesCount > 0) then
								
								(* Walk through messages *)
								repeat with m from 1 to messagesCount
									try
										set aMessage to item m of messagesArray
										set messageSentDate to date sent of aMessage
										#log "Message date: " & messageSentDate
										
										(* Check age of that date *)
										set currentDate to gCurrentDate -- current date
										set currentDateFoo to currentDate - (gMinimumAgeDays * days)
										#log "currentDateFoo: " & currentDateFoo
										if (currentDateFoo > messageSentDate) then (* Attention: "comes before" and "less than" does NOT work, only "<", see Erik's page *)
											(* Move the mail from Account's folder (IMAP) to local (ON MY MAC) *)
											tell application "Mail"
												move aMessage to mailbox (gArchiveMailboxName & "/" & accountMailboxName)
											end tell
										end if
									on error (e)
										log (e)
									end try
								end repeat (* repeat with messages *)
								
							end if (* does mailbox contain any messages? *)
						end if (* enabled? *)
						
					end repeat (* walk thru mailboxes of account *)
					
				else
					(* Too bad, this account was not enabled *)
					log "Aww, account \"" & aAccountName & "\" is disabled."
				end if (* if enabled *)
				
			end repeat (* repeat w/ accounts *)
			
		end if (* if accounts *)
		
	end tell (* tell mail *)
	
end run


(* Check wether the mailbox "parentMailbox" contains a mailbox named "childMailbox" *)
#on checkExistanceOfMailboxNameInMailbox(childMailbox, parentMailbox)
#	tell application "Mail"
#	if (parentMailbox is false) then (* also ein "root"-Mailbox *)
#		set mbs to every mailbox
#		repeat with oneMb in mbs
#			#log name of oneMb
#			if (name of oneMb is childMailbox) then
#				return true
#			end if
#		end repeat
#	else
#	set mbs to every mailbox of oneMailbox
#	repeat with oneMb in mbs
#		#log name of oneMb
#		if (name of oneMb is mailboxName) then
#			return true
#		end if
#	end repeat
#	end if
#	end tell
#	return false
#end checkExistanceOfMailboxNameInMailbox

on getEveryMailboxOfONMYMAC(argv)
	tell application "Mail"
		set mbs to every mailbox
		repeat with oneMb in mbs
			log name of oneMb
		end repeat
		return mbs
	end tell
end getEveryMailboxOfONMYMAC


on getEveryMailboxOfEveryAccount(argv)
	(* Tell mail what to do *)
	tell application "Mail"
		
		(* Get the defined accounts *)
		set accountsArray to every account
		set accountsCount to (count of accounts)
		
		(* More than 0 accounts? *)
		if (accountsCount is greater than 0) then
			
			(* Walk through accounts *)
			repeat with a from 1 to accountsCount
				
				(* Get characteristics of account *)
				set aAccount to item a of accounts
				set aAccountName to name of aAccount
				if (enabled of aAccount is true) then
					
					set boxes to mailboxes of aAccount
					log boxes
					
					repeat with box in boxes
						
						#	if (name of box is equal to mb) then
						#		set res to res + 1
						#	end if
						
					end repeat
					
				else
					(* Too bad, this account was not enabled *)
					log "Aww, account \"" & aAccountName & "\" is disabled."
				end if (* if enabled *)
				
			end repeat (* repeat w/ accounts *)
			
		end if (* if accounts *)
		
	end tell (* tell mail *)
end getEveryMailboxOfEveryAccount


to changeString of t from s to r
	set d to text item delimiters
	set text item delimiters to s
	set t to t's text items
	set text item delimiters to r
	tell t to set t to beginning & ({""} & rest)
	set text item delimiters to d
	t
end changeString

(* get a time stamp *)
on timestampWithDate(givenDate)
	set dateYear to year of givenDate
	set dateMonth to addLeadingZeros(2, (month of givenDate as number))
	set dateDay to addLeadingZeros(2, day of givenDate)
	set dateHour to addLeadingZeros(2, hours of givenDate)
	set dateMinute to addLeadingZeros(2, minutes of givenDate)
	set dateString to (dateYear & "-" & dateMonth & "-" & dateDay & "_" & dateHour & ":" & dateMinute) as string
end timestampWithDate

(* Liefert von einer Zahl n leading zeros zurck *)
on addLeadingZeros(zeroesCount, value)
	
	(* nimmt die letzten zeroesCount Stellen des Strings *)
	set newNumber to (text -zeroesCount thru -1 of ("0000000000000000" & value)) (* Da musch drauf kommen: http://macscripter.net/viewtopic.php?id=24533 *)
	log "newNumber: " & newNumber
	
	return (newNumber)
end addLeadingZeros

